package org.nuiton.topiatest;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.nuiton.topia.TopiaException;
import org.nuiton.topia.TopiaTestDAOHelper.TopiaTestEntityEnum;
import org.nuiton.topia.persistence.TopiaDAOImpl;
import org.nuiton.topia.persistence.TopiaEntity;

public abstract class CompanyDAOAbstract<E extends Company> extends TopiaDAOImpl<E> {

    @Override
    public Class<E> getEntityClass() {
        return (Class<E>)Company.class;
    }

    @Override
    public TopiaTestEntityEnum getTopiaEntityEnum() {
        return TopiaTestEntityEnum.Company;
    }

    @Override
    public void delete(E entity) throws TopiaException {
                                        {
                                        List<Department> list = getContext()
                                                    .getDAO(Department.class)
                                                    .findAllByProperties(Department.PROPERTY_COMPANY, entity);
                                            for (Department item : list) {

                                                // sletellier : Set null only if target is concerned by deletion
                                                if (entity.equals(item.getCompany())) {
                                                    item.setCompany(null);
                                                }
                        
                    }
                }

        super.delete(entity);
    }

    public E findByName(String v) throws TopiaException {
        E result = findByProperty(Company.PROPERTY_NAME, v);
        return result;
    }

    public List<E> findAllByName(String v) throws TopiaException {
        List<E> result = findAllByProperty(Company.PROPERTY_NAME, v);
        return result;
    }

    public E findBySiret(int v) throws TopiaException {
        E result = findByProperty(Company.PROPERTY_SIRET, v);
        return result;
    }

    public List<E> findAllBySiret(int v) throws TopiaException {
        List<E> result = findAllByProperty(Company.PROPERTY_SIRET, v);
        return result;
    }

    public E findContainsEmploye(Employe v) throws TopiaException {
        E result = findContains(Company.PROPERTY_EMPLOYE, v);
        return result;
    }

    public List<E> findAllContainsEmploye(Employe v) throws TopiaException {
        List<E> result = findAllContains(Company.PROPERTY_EMPLOYE, v);
        return result;
    }

    public E findContainsDepartment(Department v) throws TopiaException {
        E result = findContains(Company.PROPERTY_DEPARTMENT, v);
        return result;
    }

    public List<E> findAllContainsDepartment(Department v) throws TopiaException {
        List<E> result = findAllContains(Company.PROPERTY_DEPARTMENT, v);
        return result;
    }

    @Override
    public <U extends TopiaEntity> List<U> findUsages(Class<U> type, E entity) throws TopiaException {
        List<?> result = new ArrayList();
        List tmp;

        if (type == Department.class) {
            DepartmentDAO dao = (DepartmentDAO)
                getContext().getDAO(Department.class);
            tmp = dao.findAllByCompany(entity);
            result.addAll(tmp);
        }

        return (List<U>) result;
    }

    @Override
    public Map<Class<? extends TopiaEntity>, List<? extends TopiaEntity>> findAllUsages(E entity) throws TopiaException {
        Map<Class<? extends TopiaEntity>,List<? extends TopiaEntity>> result;
        result = new HashMap<Class<? extends TopiaEntity>, List<? extends TopiaEntity>>(1);

        List<? extends TopiaEntity> list;

        list = findUsages(Department.class, entity);
        if (!list.isEmpty()) {
            result.put(Department.class, list);
        }

        return result;
    }

} //CompanyDAOAbstract<E extends Company>
