package org.nuiton.topiatest;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.nuiton.topia.TopiaException;
import org.nuiton.topia.TopiaTestDAOHelper.TopiaTestEntityEnum;
import org.nuiton.topia.persistence.TopiaEntity;
import org.nuiton.topiatest.deletetest.Party2DAOImpl;

public abstract class PersonneDAOAbstract<E extends Personne> extends Party2DAOImpl<E> {

    @Override
    public Class<E> getEntityClass() {
        return (Class<E>)Personne.class;
    }

    @Override
    public TopiaTestEntityEnum getTopiaEntityEnum() {
        return TopiaTestEntityEnum.Personne;
    }

    public E findByName(String v) throws TopiaException {
        E result = findByProperty(Personne.PROPERTY_NAME, v);
        return result;
    }

    public List<E> findAllByName(String v) throws TopiaException {
        List<E> result = findAllByProperty(Personne.PROPERTY_NAME, v);
        return result;
    }

    public E findContainsOtherNames(String v) throws TopiaException {
        E result = findContains(Personne.PROPERTY_OTHER_NAMES, v);
        return result;
    }

    public List<E> findAllContainsOtherNames(String v) throws TopiaException {
        List<E> result = findAllContains(Personne.PROPERTY_OTHER_NAMES, v);
        return result;
    }

    public E findByGender(Gender v) throws TopiaException {
        E result = findByProperty(Personne.PROPERTY_GENDER, v);
        return result;
    }

    public List<E> findAllByGender(Gender v) throws TopiaException {
        List<E> result = findAllByProperty(Personne.PROPERTY_GENDER, v);
        return result;
    }

    public E findByOtherGender(Gender v) throws TopiaException {
        E result = findByProperty(Personne.PROPERTY_OTHER_GENDER, v);
        return result;
    }

    public List<E> findAllByOtherGender(Gender v) throws TopiaException {
        List<E> result = findAllByProperty(Personne.PROPERTY_OTHER_GENDER, v);
        return result;
    }

    public E findByAddress(Address v) throws TopiaException {
        E result = findByProperty(Personne.PROPERTY_ADDRESS, v);
        return result;
    }

    public List<E> findAllByAddress(Address v) throws TopiaException {
        List<E> result = findAllByProperty(Personne.PROPERTY_ADDRESS, v);
        return result;
    }

    public E findContainsTitle(Title v) throws TopiaException {
        E result = findContains(Personne.PROPERTY_TITLE, v);
        return result;
    }

    public List<E> findAllContainsTitle(Title v) throws TopiaException {
        List<E> result = findAllContains(Personne.PROPERTY_TITLE, v);
        return result;
    }

    @Override
    public <U extends TopiaEntity> List<U> findUsages(Class<U> type, E entity) throws TopiaException {
        return new ArrayList<U>();
    }

    @Override
    public Map<Class<? extends TopiaEntity>, List<? extends TopiaEntity>> findAllUsages(E entity) throws TopiaException {
        return new HashMap<Class<? extends TopiaEntity>, List<? extends TopiaEntity>>();
    }

} //PersonneDAOAbstract<E extends Personne>
